/*
 * Created on 22-Mar-2005
 *
 * @author David Balch david.balch@conted.ox.ac.uk
 * 
 */
package uk.ac.ox.tall.wcker.wizard;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.File;
import java.util.List;

import org.jdom.xpath.XPath;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;

import uk.ac.ox.tall.wcker.Wcker;
import uk.ac.reload.diva.util.FileUtils;
import uk.ac.reload.editor.contentpackaging.xml.ContentPackage;
import uk.ac.reload.jdom.XMLPath;
import uk.ac.reload.moonunit.schema.SchemaElement;


/**
 * SkeletonExporter is a class that deals with making a skeleton of 
 * files and directories, based on a course hierarchy and sample
 * files. 
 *  
 * @author David Balch david.balch@conted.ox.ac.uk
 *
 */
public class SkeletonExporter {
	
	NeedToKnow ntk;
	ContentPackage cp;
	File operatingDir;
	File exportDir;
	
	/**
	 * Constructor. Takes a NeedToKnow object and a export directory argument.
	 * Performs multiple file copies based on the organization structure and 
	 * the file template assigned to each item in the ntn organization.
	 * @param ntk. The NeedToKnow object which will be exported from.
	 * @param exportBase. Base directory within which the files will be exported to.
	 */
	public SkeletonExporter(NeedToKnow ntkArg, String exportDirArg) {
		if(Wcker.DEBUG > 0) {
			System.out.println("############## Started SkeletonExporter() ##############");
		}

		// Set variables
		cp = NeedToKnow.cp;
		ntk = ntkArg;
		
		operatingDir = new File(ntk.getOperatingDir() + System.getProperty("file.separator"));
		exportDir = new File(exportDirArg + System.getProperty("file.separator"));
		
		if(Wcker.DEBUG > 0) {
			System.out.println("operatingDir: " + operatingDir);
			System.out.println("exportDir: " + exportDir);
		}
		
		// Copy existing CP manifest to the export dir
		try {
			FileUtils.copyFolder(operatingDir, exportDir);
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		// Copy template files to the export dir
        XMLPath orgPath = new XMLPath("/manifest/organizations/organization");
        SchemaElement orgSchemaEl = (SchemaElement)cp.getSchemaController().getSchemaNode(orgPath);
		
		Element orgEl = cp.getElement(orgSchemaEl);
		if(Wcker.DEBUG > 1) {
			System.out.println("cp: " + cp);
			System.out.println("orgEl: " + orgEl);
		}
		copyTemplates(orgEl);
	}

	/**
	 * Build the list of files and dirctories involved in an export from
	 * the current organization.
	 */
	private void copyTemplates(Element thisItem) {
		// Look to see if a file should be copied for this item
		Element thisResource = cp.getReferencedElement(thisItem);
		if (thisResource != null) {
			// There is a relevant resource, so get details for it
			// Get destination
			File dest = new File(exportDir + System.getProperty("file.separator") + thisResource.getAttributeValue("href"));
			
			// Get source
			Element wckerInfo = thisItem.getChild("metadata", NeedToKnow.nsCP).getChild("wcker", NeedToKnow.nsWM);
			File source = new File(exportDir + System.getProperty("file.separator") + wckerInfo.getAttributeValue("templatesrc"));
			
			// If file doesn't already exist at dest, copy it.
			if (!dest.exists()) {
				//Ensure there's a parent directory to copy into
				File parent = dest.getParentFile();
				if (!parent.exists()) {
					parent.mkdirs();
				}
				// Copy and Filter file.
				// TODO
				
				// Setup input/output file streams
				FileReader inFile = null;
				FileWriter outFile = null;
				try {
					inFile = new FileReader(source);
					outFile = new FileWriter(dest);
				} catch (IOException e) {
					e.printStackTrace();
				}
				
				// BufferedReader to get lines at a time
				BufferedReader in = new BufferedReader(inFile);
				
				// Get filter information for this file
	    		XPath filterPath;
	    		Element filterEL = null;
	    		try {
	    			filterPath = XPath.newInstance("cp:metadata/wm:wcker/wm:filter");
	    			filterPath.addNamespace(NeedToKnow.nsCP);
	    			filterPath.addNamespace(NeedToKnow.nsMD);
	    			filterPath.addNamespace(NeedToKnow.nsWM);
	    			filterEL = (Element)filterPath.selectSingleNode(thisItem);
	    		} catch (JDOMException e) {
	    			e.printStackTrace();
	    		}

	    		List filters = filterEL.getChildren();
				//System.out.println("filters: " + filters);
				
				// Filter it and write to dest file
				String line;
				try {
					while ((line = in.readLine()) != null) {
						if (line.contains("<%")) {
							// Line contains the opening filter token, so lets filter it
							
							// Apply title filter
							line = line.replaceAll("<%title%>", thisItem.getChildText("title", NeedToKnow.nsCP));

							// Apply other filters
							for (int f=0; f<filters.size(); f++) { 
								Element thisFilter = (Element)filters.get(f);
								String tokenReplace = thisFilter.getValue();
								String filterToken = "<%" + thisFilter.getName() + "%>";

								//System.out.println("filterToken: " + filterToken);
								//System.out.println("tokenReplace: " + tokenReplace);
								line = line.replaceAll(filterToken, tokenReplace);
							}
						}
						outFile.write(line);
					}
					in.close();
					outFile.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
				
			}
		}
		
		// Iterate over any child items
		List childElements = thisItem.getChildren("item", NeedToKnow.nsCP);
		
		//Iterate over the WckerTree, adding each file and directory to the List
		for(int i = 0; i < childElements.size(); ++i) {
			//Isolate the Element we're dealing with
			Element thisChild = (Element)childElements.get(i);
			//Iterate through the tree
			copyTemplates(thisChild);
		}
		
		// Copy support files.
		List copyNodes = NeedToKnow.supportFiles.getChildren("copy", NeedToKnow.nsWSI);
					
		try {
			for (int i=0; i<copyNodes.size(); i++) {
				Element thisCopy = (Element)copyNodes.get(i);
				File inFile = new File(operatingDir.getAbsolutePath() + System.getProperty("file.separator") + thisCopy.getAttributeValue("source"));
				File outFile = new File(exportDir.getAbsolutePath() + System.getProperty("file.separator") + thisCopy.getAttributeValue("dest"));

				//System.out.println("inFile: " + inFile);
				//System.out.println("outFile: " + outFile);
				
				if (inFile.isFile()) {
					FileUtils.copyFile(inFile, outFile);
				} else if (inFile.isDirectory()) {
					FileUtils.copyFolder(inFile, outFile);
				}
			}

		} catch (Exception e) {
		}
	}
}