/**
 *  RELOAD TOOLS
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Paul Sharples
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Paul Sharples
 *  e-mail:   p.sharples@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.scormplayer;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import javax.xml.parsers.ParserConfigurationException;
import org.jdom.JDOMException;
import org.xml.sax.SAXException;
import uk.ac.reload.dweezil.gui.DweezilProgressMonitor;
import uk.ac.reload.dweezil.util.NativeLauncher;
import uk.ac.reload.diva.util.FileUtils;
import uk.ac.reload.diva.util.ZipUtils;
import uk.ac.reload.moonunit.contentpackaging.CP_Core;
import uk.ac.reload.scormplayer.client.generic.contentpackaging.DocumentHandlerException;
import uk.ac.reload.scormplayer.client.generic.contentpackaging.NoItemFoundException;
import uk.ac.reload.scormplayer.client.generic.contentpackaging.ScormPackageHandler;
import uk.ac.reload.scormplayer.client.generic.prefs.ScormPlayerPrefs;
import uk.ac.reload.scormplayer.client.generic.schemavalidation.SchemaValidationException;
import uk.ac.reload.scormplayer.client.generic.schemavalidation.SchemaValidator;
import uk.ac.reload.scormplayer.server.tomcat.ScormTomcatHandler;


/**
 * Control station.  All of the methods required to import/export
 * start/reset/stop packages and also to start/stop the server either via
 * the gui interfaces or via calls from another java app.
 * @author Paul Sharples
 * @version $Id: ControlStation.java,v 1.6 2004/07/19 16:10:15 ps3com Exp $
 */

public class ControlStation implements IControlStation {
	
    /**
     * A single instance of this class.
     */
    protected static ControlStation _sharedInstance;    
    
    /**
     * constructor
     */
    private ControlStation(){}
    
    /**
     * Get the single instance
     */
    public static ControlStation getSharedInstance() {
	    if(_sharedInstance == null) _sharedInstance = new ControlStation();
	    return _sharedInstance;
	}
    
	/*
	 ***************************************************************************
	 *   Methods for importing, validation and manipulation of content packages.
	 *   These methods are wrapped in this class, so they can be used by both
	 *   the GUI in MainWindow and also called directly here, so the same things
	 *   can be done without having to use the GUI interface.
	 ***************************************************************************
	 */
	
	
	/**
	 * Method to ascertain if manifest is valid
	 * @return true or false denoting if manifest is valid
	 */
	public boolean isValidManifest(File file) throws DocumentHandlerException, JDOMException, IOException  {
		File manifestFile = null;
	    
	    if(isZip(file)) {
			// need to grab the manifest out of the zip file
	        File tempFile = File.createTempFile("~rld-", ".xml");
	        // Make sure we delete it on exit
	        tempFile.deleteOnExit();
	        manifestFile = findManifest(file, tempFile);
	        if(manifestFile == null) return false;
		}
		else {
		    manifestFile = file;
		}
	    ScormPackageHandler handler = new ScormPackageHandler(manifestFile);
		return handler.isSCORM12Manifest();
	}
	
	/**
	 * Manifest Parser - used to obtain info about organizations and ascertain
	 * which item reference SCOs/Assets.  Calls ScormPackageHandler to do this
	 * @param aFile - file reference to the manifest
	 */
	public void parseManifest(File man) throws NoItemFoundException, JDOMException, IOException {
	    ScormPackageHandler handler = new ScormPackageHandler(man);
		handler.buildSettings();		
	}	
	
	public void setupValidateAgainstSchema(String schemaName,
            File manifest, String projectName) throws Exception {

        File FIXED_SCHEMA_FOLDER = new File(ScormPlayerPrefs.getInstance()
                .getPrefsFolder(), "schema" + File.separatorChar
                + "validationschemas");
        // the folder to copy it to
        File SOURCE_SCHEMA_FOLDER = new File(ScormTomcatHandler
                .getSharedInstance().getCoursesPath(), projectName);

        validatePackageAgainstSchema("ims_xml.xsd", SOURCE_SCHEMA_FOLDER,
                FIXED_SCHEMA_FOLDER, manifest);

    }
	
	public void validatePackageAgainstSchema(String schemaName, File destinationFolder, File sourceFolder, File manifest)
							throws SchemaValidationException, IOException, SAXException, ParserConfigurationException {
		replaceSchema(schemaName, destinationFolder, sourceFolder);
		SchemaValidator validator = new SchemaValidator(manifest);
		validator.validate();
		if (validator.hasErrors()){
			String allMessages = "";
			String[] messages = validator.getErrorMessages();
			for(int i = 0; i < messages.length; i++){
				allMessages += messages[i] + System.getProperty("line.separator");
			}
			throw new SchemaValidationException(allMessages);
		}
	}	
	
	public boolean replaceSchema(String schemaName, File destinationFolder, File sourceFolder) throws IOException {
		File originalSchema = new File(destinationFolder.toString() + File.separatorChar + schemaName);
		File newSchema = new File(sourceFolder.toString() + File.separatorChar + schemaName);		
		if (originalSchema.exists() && newSchema.exists()){
			FileUtils.copyFile(newSchema, originalSchema);
			// schema copied over okay...
			return true;
		}
		else{
			// one of the schemas doesnt exist...
			return false;
		}
	}	
	
	/**
	 * Method to ascertain if a file reference is a ZIP file
	 * @param aFile
	 * @return true or false
	 */
	public boolean isZip(File aFile) {
		if (aFile.getName().toLowerCase().endsWith(ScormPackageHandler.ARCHIVE_EXTENSION))
			return true;
		else
			return false;
	}
	
	/**
	 * Method to check if imsmanifest.xml file exists in a zip file.
	 * @param zipFile
	 * @returns the file if found, null if not found
	 */
	public File findManifest(File zipFile, File tempFile) throws FileNotFoundException, IOException {
		return ZipUtils.extractZipEntry(zipFile, CP_Core.MANIFEST_NAME, tempFile);
	}
	
	/**
	 * Calls ZipUtils() Unzips an archive into a specified folder
	 * @param file
	 * @param destinationFolder
	 * @param mon
	 * @return true or false if successful
	 */
	public boolean unpackZip(File file, File destinationFolder, DweezilProgressMonitor mon) throws IOException {	    
		return ZipUtils.unpackZip(file, destinationFolder, mon);
	}
	
	/**
	 * Method to check to see if a particular project folder already exists
	 * under the web server.
	 * @param projectFolderName
	 * @return either NULL if it exists already or a new File reference if doesnt
	 */
	public File checkProjectFolderAlreadyExists(String projectFolderName) {
	    File newProjectFolder = new File(ScormTomcatHandler.getSharedInstance().getCoursesPath(), projectFolderName);	    
		if (newProjectFolder.exists()){
			return null;
		}
		else {
			return newProjectFolder;
		}
	}
	
	/*
	 *******************************************************************************
	 * Methods for starting/stopping the web server and playing packages
	 *******************************************************************************
	 */
	
	/**
	 * Method to start the web server
	 */
	public void startServer() throws Exception {			    
		ScormTomcatHandler.getSharedInstance().startServer();		
	}
		
	/**
	 * Method to ascertain if server is already running
	 * @return true or false if server is running or not
	 */
	public boolean isAlive() {
		return ScormTomcatHandler.getSharedInstance().isAlive();
	}
		
	/**
	 * Method to stop web server
	 */
	public void stopServer() throws Exception {	
		ScormTomcatHandler.getSharedInstance().stopServer();
	}
		
	/**
	 * Method to launch a scorm package
	 * @param thePackage - which package to launch
	 */
	public void startPackage(String thePackage) {	    
	    NativeLauncher.launchURL(ScormTomcatHandler.getSharedInstance().getLaunchURL(thePackage));
	}
	
	/**
	 * MEthod to reset a package to its default settings - as if it hadn't been played yet.
	 * @param path
	 * @throws NoItemFoundException
	 */
	public void resetPackage(File path) throws NoItemFoundException, JDOMException, IOException {	
			File man = new File(path, CP_Core.MANIFEST_NAME);
			ScormPackageHandler handler = new ScormPackageHandler(man);
			handler.buildSettings();	
	}
}