/**
 *  RELOAD TOOLS
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Paul Sharples
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Paul Sharples
 *  e-mail:   p.sharples@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.scormplayer.client.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.IOException;
import java.util.Vector;

import javax.swing.ImageIcon;
import javax.swing.JDesktopPane;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.UIManager;

import uk.ac.reload.dweezil.gui.DweezilFileChooser;
import uk.ac.reload.dweezil.util.DweezilUIManager;
import uk.ac.reload.dweezil.util.UIUtils;
import uk.ac.reload.dweezil.gui.ErrorDialogBox;
import uk.ac.reload.dweezil.gui.SplashScreen;
import uk.ac.reload.dweezil.gui.StatusWindow;
import uk.ac.reload.diva.util.CopyTask;
import uk.ac.reload.dweezil.util.OSXApplication;
import uk.ac.reload.dweezil.util.NativeLauncher;

import uk.ac.reload.scormplayer.client.generic.prefs.ScormPlayerPrefs;
import uk.ac.reload.scormplayer.client.gui.mainwindow.MainWindow;
import uk.ac.reload.scormplayer.client.gui.menu.MainMenu;
import uk.ac.reload.scormplayer.client.gui.menu.MenuAction_About;
import uk.ac.reload.scormplayer.client.gui.prefs.PrefsDialog;
import uk.ac.reload.scormplayer.properties.ScormPlayerProperties;


/**
 * The ScormPlayerFrame Frame - the main graphical application. Builds the GUI
 * 
 * @author Phillip Beauvoir
 * @author Paul Sharples
 * @version $Id: ScormPlayerFrame.java,v 1.10 2004/12/03 21:25:03 phillipus Exp $
 */
public class ScormPlayerFrame
extends JFrame
implements OSXApplication, IIcons {
	
    /**
     * If running locally, this is the path relative to your working dir of the support jar
     * The support jar contains server files, and help
     */
    public static final String SUPPORT_JAR_LOCATION = "reload-support.jar";

    /**
     * If running under webstart, retrieving this resource will allow you to locate the SUPPORT_JAR_LOCATION
     * The support jar contains server files, and help
     */
    public static final String SUPPORT_JAR_RESOURCE = "server";

    /**
	 * The Status Window
	 */
	private StatusWindow _statusWindow;
	
	/**
	 * The singleton instance of the app
	 */
	private static ScormPlayerFrame _instance;
	
	/**
	 * The desktop pane.
	 */
	protected JDesktopPane _desktop;
	
	/**
	 * The main internal window
	 */
	protected static MainWindow _mainWindow;
	
	/**
	 * Gets the Singleton Instance of the Editor
	 * 
	 * @return The Singleton Instance of the Editor
	 */
	public static ScormPlayerFrame getInstance() {
		return _instance;
	}
	
	/**
	 * @return The Status Window
	 */
	public StatusWindow getStatusWindow() {
		return _statusWindow;
	}
	
	/**
	 * Gets the Singleton Instance of the MainWindow
	 * 
	 * @return - singleton instance of the mian window
	 */
	public static MainWindow getMainWindow() {
		return _mainWindow;
	}
	
	/**
	 * @return the Help File
	 */
	public File getHelpFile() {
		return ScormPlayerProperties.getFileProperty("help.file");
	}
	
	/**
	 * Show the Help File
	 */
	public void showHelp() {
		File file = getHelpFile();
		if(file != null) {
		    NativeLauncher.launchFile(file);
		}
	}
	
	/**
	 * Constructor method for new SCORM Viewer
	 */
	public ScormPlayerFrame() {
		_instance = this;
		
		// Set Properties file
		System.setProperty("scormplayer.properties.file", "uk.ac.reload.scormplayer.properties.rb");

		// Turn off Java 1.4.2 Windows XP L&F - it's bad
		System.setProperty("swing.noxp", "true");

		// UI Defaults
		DweezilUIManager.setMacUIDefaults(ScormPlayerProperties.getString("APP_NAME"), getInstance());
		
		// SplashScreen
		SplashScreen splash = new SplashScreen(DweezilUIManager.getIcon(IMAGE_SPLASH));
		
		boolean DEBUG = ScormPlayerProperties.getString("DEBUG").equals("true");
						
		// Get the Status Window going
		_statusWindow = new StatusWindow(ScormPlayerProperties.getString("APP_NAME"),
		        ScormPlayerProperties.getString("VERSION"),
		        ScormPlayerProperties.getString("BUILD_DATE"),
				DweezilUIManager.getIcon(ICON_APP32),
				!DEBUG, ScormPlayerPrefs.getInstance().getPrefsFolder());
		
		
		// Output some useful information
		System.out.println("Preferences folder:\t" + ScormPlayerProperties.getFileProperty("prefs.dir"));
		System.out.println("Server folder:\t\t" + ScormPlayerProperties.getFileProperty("server.dir"));
		System.out.println("Tomcat Version:\t\t" + ScormPlayerProperties.getString("SERVER_INFO"));
				
        // Check that the prefs (and helpers) folder is in working order
        boolean checkSupport = ScormPlayerPrefs.getInstance().getBooleanValue(ScormPlayerPrefs.GENERAL_CHECK_SUPPORT);
        if(checkSupport) checkSupportFolder();
               
        // Set Look And Feel from User Prefs
		String lf = ScormPlayerPrefs.getInstance().getValue(ScormPlayerPrefs.GENERAL_PREFS_LF);
		if(lf == null) lf = UIManager.getSystemLookAndFeelClassName();
		DweezilUIManager.setLookAndFeel(lf, new Component[]{getInstance(), getStatusWindow()});
		
		// Set default folder from User Prefs
		String defFolder = ScormPlayerPrefs.getInstance().getValue(ScormPlayerPrefs.GENERAL_PREFS_DEFAULT_FOLDER);
		if(defFolder != null) DweezilFileChooser.setDefaultFolder(new File(defFolder));
		
		// Set Logging to highest level if not debug mode
		if(!DEBUG) {
		    System.setProperty("org.apache.commons.logging.Log", "org.apache.commons.logging.impl.SimpleLog");
		    System.setProperty("org.apache.commons.logging.simplelog.defaultlog", "fatal"); 
		}
				
		// assign our main window var
		_mainWindow = new MainWindow();
		
		// Set application title in title bar
		super.setTitle(ScormPlayerProperties.getString("APP_NAME"));
		
		// Set application Icon
		ImageIcon icon = DweezilUIManager.getIcon(ICON_APP32);
		if(icon != null) setIconImage(icon.getImage());
		
		// Trap window closing event for our Window listener
		setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		
		// Add a listener to Close our window down on exit
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				closeApp();
			}
		});
		
		// Set up desktop
		JPanel panel = new JPanel(new BorderLayout());
		_desktop = new JDesktopPane();
		panel.add(_desktop, BorderLayout.CENTER);
		setContentPane(panel);
		
		// Create Menu Bar (after setting up the desktop, please!)
		setJMenuBar(MainMenu.sharedInstance);
		
		// Create Toolbar
		getContentPane().add(MainMenu.sharedInstance.getToolBar(), BorderLayout.NORTH);
		
		// add the main window
		getContentPane().add(_mainWindow, BorderLayout.CENTER);
		
	    splash.close();
		
		// Show the main Frame Window, centred
	    UIUtils.centreWindowProportional(this, 0.9, 0.9);
		setVisible(true);
	}
	
	////////////////////////////////////////////////////////////////////////////
	////////////////////////////////////////////////////////////////////////////
	////////////////////////////////////////////////////////////////////////////
	
	/**
	 * Main program entry point
	 * 
	 * @param args Command line arguments
	 */
	public static void main(String[] args) {
	    // Check for some args for over-riding system properties
        // User can create a shortcut to the application and add a switch to
	    // the end of the command line such as:
        // user.home=mydir
        // If there are spaces in the value, use quotes like:
        // "user.home=d://some dir"
	    for(int i = 0; i < args.length; i++) {
	        int pos = args[i].indexOf("=");
	        if(pos != -1) {
	            try {
	                String key = args[i].substring(0, pos).trim();
	                String value = args[i].substring(pos + 1).trim();
	                System.setProperty(key, value);
	            }
	            catch(IndexOutOfBoundsException ex) {
	            }
	        }
        }
	    
		new ScormPlayerFrame();
	}
	
	/**
	 * Quit the application Put any cleanup code here.
	 */
	public void closeApp() {
		// Tell any registered listeners
		boolean result = fireApplicationClosing();
		if(result) {

        	// Save Prefs again.  This saves most recent list and other things.
        	try {
                ScormPlayerPrefs.getInstance().save();
            }
            catch(IOException ex) {
                ErrorDialogBox.showWarning(this, "Could not save Preferences: ", "Preferences", ex);
            }

			// Quit
			System.exit(0);
		}
	}
	
    /**
     * make sure the prefs and all support files are in working order
     */
    private void checkSupportFolder() {
        CopyTask copyTask = null;
        try {
            // Local jar while developing
            copyTask = new CopyTask(SUPPORT_JAR_LOCATION, ScormPlayerPrefs.getInstance().getPrefsFolder());
        } catch (IOException e) {
            // Jar in classpath
            try {
                copyTask = new CopyTask(SUPPORT_JAR_RESOURCE, ScormPlayerPrefs.getInstance().getPrefsFolder());
            } catch (Exception e1) {
                e1.printStackTrace();
                System.err.println("If you're running locally, make sure that you have " +
                        "a scorm-support.jar in your build directory (" + System.getProperty("user.dir") + "/" + SUPPORT_JAR_LOCATION + "). " +
                        "You can build this jar using the build-support jar target in the ant build.xml file.");
                return;
            }
        }
        final CopyTask copyTask1 = copyTask;
        new Runnable() {
            public void run() {
                // we only need to backup if we're worried that the copy doesn't do it's job properly
                // or if the user has changed the default helper files and wants to preserve those changes
                // copyTask1.backup();
                try {
                    copyTask1.execute(true);
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }.run();
    }

   
   
    
    /**
	 * Set the Title Bar String
	 * 
	 * @param title The Title Bar String
	 */
	public void setTitle(String title) {
		super.setTitle(ScormPlayerProperties.getString("APP_NAME") + " - " + title);
	}
	
	// Implement OSXApplication
	public void preferences() {
		PrefsDialog.getSharedInstance().show();
	}

	public void about() {
		MenuAction_About.doShowMessageDialog();
	}

	public void quit() {
		closeApp();
	}

	public OSXApplication getAppInstance() {
		return getInstance();
	}

	////////////////////////////////////////////////////////////////////////////
	//   APPLICATION LISTENERS ////////////////////////////////////////////////
	////////////////////////////////////////////////////////////////////////////
	
	/**
	 * List of Listeners
	 */
	private Vector listeners = new Vector();
	
	/**
	 * Add an Application Listener
	 * 
	 * @param listener
	 *            The Application Listener
	 */
	public synchronized void addApplicationListener(ApplicationListener listener) {
		if(!listeners.contains(listener))listeners.addElement(listener);
	}
	
	/**
	 * Remove an Application Listener
	 * 
	 * @param listener
	 *            The Application Listener
	 */
	public synchronized void removeApplicationListener(ApplicationListener listener) {
		listeners.removeElement(listener);
	}
	
	/**
	 * Tell our listeners that we are closing the App They must reply by
	 * answering with true for OK, or false for not OK
	 */
	public boolean fireApplicationClosing() {
		boolean ok = true;
		// Go from last to first in case anyone removes themselves
		for(int i = listeners.size() - 1; i >= 0; i--) {
			ApplicationListener listener = (ApplicationListener) listeners.elementAt(i);
			boolean result = listener.applicationClosing();
			if(result == false) ok = false;
		}
		return ok;
	}
}