/**
 *  RELOAD TOOLS
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Paul Sharples
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.scormplayer.server.manager;

import java.io.File;
import java.util.Iterator;
import java.util.Vector;
import org.jdom.Element;
import uk.ac.reload.diva.util.GeneralUtils;
import uk.ac.reload.moonunit.contentpackaging.CP_Core;
import uk.ac.reload.scormplayer.properties.ScormPlayerProperties;
import uk.ac.reload.scormplayer.server.sequence.SequenceManager;
import uk.ac.reload.scormplayer.server.tomcat.ScormTomcatHandler;
import uk.ac.reload.scormplayer.client.generic.contentpackaging.NavigationViewer;

/**
 * A class used to act as the main server object, which holds references to
 * each items found within the manifest.
 * It is mainly used to build the navigation for the tree widget
 * and ascertain what items are valid, if they point to something, etc.  It
 * builds a model of this from the the xml navigation file that was created as
 * a package was imported.  It is also responsible for maintaining the nav file
 * updating it accordingly when a user has completed an item
 * 
 * @author Paul Sharples
 * @version $Id: ScormManager.java,v 1.8 2004/07/19 16:18:55 ps3com Exp $
 */
public class ScormManager {
    
    /**
     * The package name
     * Note - this should not really be static - this assumes you're only ever going to
     * have one package open at a time.
     */
    public static String PACKAGE_NAME;
   
    /**
     * instance of the navigationviewer - used to lookup values/elements from the manifest.
     */
    protected NavigationViewer _navViewer;
    
    /**
     * A boolean to tell us if the player should show the tree widget
     */
    private boolean _showTreeWidget;
    
    /**
     * A boolean to tell us if the player should show the next and prev navigation
     */
    private boolean _showNavigation;
        
    /**
     * Whether to use auto progression thru the package
     */
    private boolean _autoProgression;
    
    /**
     * The manager's instance of the scorm sequence manager - used for sequencing
     * and prerequisites, and cmi updating models
     */
    private SequenceManager _sequence;
    
    /**
     * boolean value used to flag that navigation nav file has been found
     */
    private boolean _error_found;
    
    private boolean _ReloadNoItemFoundExceptionFlag = true;
    /**
     * A count used to order the items in an org
     */
    private int sequenceCount = 0;
    
    /**
     * An array of javascrtip string info which the tree widget will use
     */
    private String[] _javascriptStrings;
    
    /**
     * Keep hold of the default org (JDOM) element
     */
    private Element _defaultorg;
    
    /**
     * A count of organizations (should really only be one for the tree widget)
     */
    private int _orgCount = -1;
	
	/**
	 * Used to keep track of how many items there are in a organization
	 * (this is array based, so initially set to -1)
	 */    
	private int _itemCount = -1; 
		
	/**
     * Default constructor
     */
    public ScormManager() {}
	        
    /**
     * Alternate constructor - used to pass the the package name (i.e. - used
     * to construct a path to disk and find the files needed.  It also allows
     * us to specify if we want to show the tree widget and the next and previous
     * buttons in the navigation
     */
    public ScormManager(String packageName, boolean showTreeWidget,
            boolean showNavigation, boolean autoProgression) {
        _error_found = false;
        File pathToNavFile = ScormTomcatHandler.getSharedInstance()
                .getSettings(packageName);
        // Make sure its there
        if(pathToNavFile.exists()) {
            try {
                _navViewer = new NavigationViewer(pathToNavFile);
            }
            catch(Exception ex) {
                System.out.println("Could not load manifest");
                if(ScormPlayerProperties.getString("DEBUG").equals("true")) {
                    ex.printStackTrace();
                }
            }
            // Package Links - use a clone so we can add referenced elements
            Element manifestRoot = (Element)_navViewer.getRootElement().clone();
            // now get the organizations node
            Element orgs = manifestRoot.getChild(CP_Core.ORGANIZATIONS,
                    manifestRoot.getNamespace());
            // remember the default organization
            _defaultorg = _navViewer.getDefaultOrgElement(orgs);
            // NO orgs, so cant play - flag it
            if(_defaultorg == null) {
                _ReloadNoItemFoundExceptionFlag = false;
            }
            else {
	            PACKAGE_NAME = packageName;
	            _showNavigation = showNavigation;
	            _showTreeWidget = showTreeWidget;
	            _autoProgression = autoProgression;
	            _sequence = new SequenceManager(PACKAGE_NAME, _defaultorg
	                    .getAttributeValue(CP_Core.IDENTIFIER));
	            // it exists so try to load it...
	            try {
	                initialize();
	            }
	            catch(Exception ex) {
	                System.out.println("Could not initialise manifest");
	                if(ScormPlayerProperties.getString("DEBUG").equals("true")) {
	                    ex.printStackTrace();
	                }
	                _error_found = true;
	            }
            }     //
        } 
        else {
            // problems if we cannot find the nav file...
            _error_found = true;
        }
    }
    
    /**
     * Accessor method to obtain the array of javascrtip strings needed by the
     * tree widget
     * 
     * @return
     */
    public String[] getJavascriptStrings(){
        return _javascriptStrings;
    }
     
    /**
     * Accesor method for the sequencer
     * @return
     */
    public SequenceManager getSequence(){
    	return _sequence;
    }
           
    /**
     * Accessor method - should we show the tree or not?
     * @return boolean
     */
    public boolean isTreeVisible(){
        return _showTreeWidget;
    }
    
    /**
     * Accessor method - should we show the navigation or not?
     * @return boolean
     */
    public boolean isNavVisible(){
        return _showNavigation;
    }
    
    /**
     * Accessor method - boolean to use auto progression of items in player
     * @return
     */
    public boolean isAutoProgressionEnabled(){
        return _autoProgression;
    }   
    
    /**
     * A method to do some initialising before we actually play the package.
     * Here we get the default organization and then pass that to a method
     * which will parse that organization (JDOM) and figure out what we need,
     * in terms of items, sequence, prerequisites, etc
     */
    public void initialize(){
        if (_navViewer.getDocument() == null){
            return;
        }        
        _javascriptStrings = parse(false);
    }
        
    /**
     * A method to start the parse of the manifest and write the javascript strings
     * which the tree widget needs and also to populate the sequencer
     * @param useRelativePaths
     * @return
     */
    protected String[] parse(boolean useRelativePaths) {
	    // New Vector
	    Vector v = new Vector();		      		      	    
	    writePackageSettings(v, "packageName", getProjectName());	    	   
	    // now call createNavLinks() which should interrogate the org/item structure	    	    
	    createNavLinks(v, _defaultorg, "menu", useRelativePaths);	    
	    // Convert Vector to String array
	    String[] javascriptStrings = new String[v.size()];
	    v.copyInto(javascriptStrings);
	    return javascriptStrings;
	}
 
    /**
     * Recursive method to find all of the javascript strings (item info)
     * and populate the sequencer
     * @param javascriptStrings
     * @param element
     * @param menuParent
     * @param useRelativePaths
     */
    protected void createNavLinks(Vector javascriptStrings, Element element, String menuParent, boolean useRelativePaths) {
		String name = element.getName();		
		// ORGANIZATION    	
		if(name.equals(CP_Core.ORGANIZATION) && _navViewer.isDocumentNamespace(element)) {
			_itemCount = -1;
			++_orgCount;
			String orgId = element.getAttributeValue(CP_Core.IDENTIFIER);    		
			menuParent = "menu";
			String title = "Organization";
			// Display Title if there is one
			Element titleElement = element.getChild(CP_Core.TITLE, element.getNamespace());
			if(titleElement != null) {
				if(!titleElement.getText().equals("")) title = titleElement.getText();
			}
			// find out if this the default organization...
			String _defaultOrganization = _defaultorg.getAttributeValue(CP_Core.IDENTIFIER);			
			if (_defaultOrganization != null){
				if (_defaultOrganization.equals(orgId)){
					writePackageSettings(javascriptStrings, "_defaultOrg", _orgCount);
				}
			}    		
			writeOrganization(javascriptStrings, title, orgId);
		}    			
		// ITEM
		else if(name.equals(CP_Core.ITEM) && _navViewer.isDocumentNamespace(element)) {
			++_itemCount;			
			String itemId = element.getAttributeValue(CP_Core.IDENTIFIER);
			String hyperLink = "";
		    String url = "";
			String scoType = "";
			// Display Title if there is one
			String title = "Item";
			Element titleElement = element.getChild(CP_Core.TITLE, element.getNamespace());
			if(titleElement != null) {
				if(!titleElement.getText().equals("")) title = titleElement.getText();
			}			
			// check to see that the isvisible attribute is not set to false...
			String isVisibleAttrib = element.getAttributeValue(CP_Core.ISVISIBLE);
			if(isVisibleAttrib != null) {
				if(isVisibleAttrib.equals("false")){ 
				    title = "* hidden";
				}
			}			
			// What does this Item reference?
			Element ref_element = _navViewer.getScormCore().getReferencedElement(element);			
			String prerequisites = "";			
			if(ref_element != null) {
				String ref_name = ref_element.getName();				
				// A RESOURCE
				if(ref_name.equals(CP_Core.RESOURCE)) {				    
				    scoType = _navViewer.findScoType(element);
				    // Relative path for export - Note the "../" is relative to where the Nav file is!
				    if(useRelativePaths) {
				        url = _navViewer.getScormCore().getRelativeURL(element);
				        // Only if local path add relative bit
				        if(GeneralUtils.isExternalURL(url) == false) {
				            url = "../" + url;
				        }
					}
				    // Absolute Paths for Previewing in-situ
					else {
					    String turl = _navViewer.getLaunch(element);
					    url = turl;					    					    
					}				    				    				    
					if(url != null) {
					    hyperLink = url;					 
					    if(!title.equals("* hidden")){					       
			                prerequisites = _navViewer.getPrerequisites(element);
			                if (prerequisites == null){
			                    prerequisites = "";
			                }                
					    }
					}
				}				
				// A sub-MANIFEST
				else if(ref_name.equals(CP_Core.MANIFEST)) {
				    hyperLink = "javascript:void(0)";
					// Get ORGANIZATIONS Element
					Element orgsElement = ref_element.getChild(CP_Core.ORGANIZATIONS, ref_element.getNamespace());
					// Now we have to get the default ORGANIZATION
					if(orgsElement != null) ref_element = _navViewer.getScormCore().getDefaultOrganization(orgsElement);					
					// Get the children of the referenced <organization> element and graft clones
					if(ref_element != null) {
						Iterator it = ref_element.getChildren().iterator();
						while(it.hasNext()) {
							Element ref_child = (Element) it.next();
							element.addContent((Element) ref_child.clone());
						}
					}
				}
			}			
			else {
			    hyperLink = "javascript:void(0)";			    
			}
			//System.out.println("adding to sequencer:"+ itemId + " " + hyperLink+ " " + _itemCount+ " " + scoType+ " " +title+ " " + prerequisites);
		    _sequence.addNewItem(itemId, hyperLink, _itemCount, scoType, title, prerequisites);
			writeItem(javascriptStrings, title, hyperLink, itemId, menuParent);
			menuParent = itemId;
		}   		
		// round we go again...
		Iterator it = element.getChildren().iterator();
		while(it.hasNext()) {
			Element child = (Element) it.next();    		
			createNavLinks(javascriptStrings, child, menuParent, useRelativePaths);
		}
	}
    
    /**
	 * Writes "top level" CPAPI settings to the vector of javascript code.
	 * @param name - the name of the item to set
	 * @param value - the value of the item to set (String)
	 */
	protected void writePackageSettings(Vector javascriptStrings, String name, String value) {
		javascriptStrings.add("CPAPI." + name + " = \"" + value + "\";");
	}
	
	/**
	 * Writes "top level" CPAPI settings to the vector of javascript code.
	 * @param name - the name of the item to set
	 * @param value - the value of the item to set (int)
	 */
	protected void writePackageSettings(Vector javascriptStrings, String name, int value) {
		javascriptStrings.add("CPAPI." + name + " = " + value + ";");
	}
	
	/**
	 * Writes "organization" CPAPI settings to the vector of javascript code.
	 * @param title - the title of this organization
	 * @param orgId - the organization identifier
	 */
	protected void writeOrganization(Vector javascriptStrings, String title, String orgId) {
		javascriptStrings.add("CPAPI.orgArray(" + _orgCount + ").organizationName = \"" + escapeQuotes(title) + "\";");
		javascriptStrings.add("CPAPI.orgArray(" + _orgCount + ").organizationIdentifier = \"" + orgId + "\";");    	
	}
	
	/**
	 * Writes "item" CPAPI settings to the vector of javascript code.
	 * @param title - the title of this item
	 * @param url - the launch url of this item    
	 * @param itemId - the item identifier
	 * @param parentMenu -  the tree node to attach this to.
	 */   
	protected void writeItem(Vector javascriptStrings, String title, String url, String itemId, String parentMenu) {
		// the javscript tree widget doesn't like hyphens, so replace them with underscores...
		parentMenu = parentMenu.replace('-', '_');    	
		// add the item...
		javascriptStrings.add("CPAPI.orgArray(" + _orgCount + ").itemArray(" + _itemCount + ").itemTitle = \"" + escapeQuotes(title) + "\";");
		javascriptStrings.add("CPAPI.orgArray(" + _orgCount + ").itemArray(" + _itemCount + ").itemIdentifier = \"" + itemId + "\";");
		javascriptStrings.add("CPAPI.orgArray(" + _orgCount + ").itemArray(" + _itemCount + ").itemParent = \"" + parentMenu + "\";");
		javascriptStrings.add("CPAPI.orgArray(" + _orgCount + ").itemArray(" + _itemCount + ").itemHyper = \"" + url + "\";");    	
	}
	
	/**
	 * Utility method to escape all string delimiters found with a string
	 * @param inputString - the string to escape
	 * @return - the string with escaped quotes (in javascript format)
	 */
	protected String escapeQuotes(String inputString) {
		inputString = inputString.replaceAll("'", "\\\\\\\\'"); // replace a single quote with \\'                 
		inputString = inputString.replaceAll("\"","\\\\\""); // replace a double quote with \"
		return inputString;
	}
    
    /**
     * Accessor method - get the project name
     * @return - a string
     */
    public String getProjectName() {
        return PACKAGE_NAME;
    }
    
    
    /**
     * @return Returns the _errorfound.
     */
    public boolean isErrorFound() {
        return _error_found;
    }
    
    public boolean hasItemsFound(){
        return this._ReloadNoItemFoundExceptionFlag;
    }
}