/**
 *  RELOAD TOOLS
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Paul Sharples
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Paul Sharples
 *  e-mail:   p.sharples@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.scormplayer.server.sequence;

import uk.ac.reload.scormplayer.server.servermodels.ScoDocument;
import uk.ac.reload.moonunit.contentpackaging.SCORM12_Core;
/**
 * A class used to house a single item from a scorm package.  It should
 * only contain items that actually reference something. Here we store
 * data about this particular item - its launch, its title, its ID, etc
 * Also this class associates a ScoDocument with this item, but only if
 * its a sco, not for assets.  A ScoDocument looks after the persisting
 * of the CMI datamodel back to xml, and also reading it back into the javascript
 * server model.
 * 
 * @author Paul Sharples
 * @version $Id: ItemSequence.java,v 1.2 2004/07/19 16:05:08 ps3com Exp $
 */
public class ItemSequence {

    // Here we store some information about this item
    private boolean _hasPrerequisites;
    private String _prerequisites;
    private String _scoitemID;
    private String _title;
    private String _launchUrl;
    private int _sequence;
    private String _scormType;
    // If this item is a sco, then we will associate it with a ScoDocument
    private ScoDocument _scoDataModel;

    /**
     * Constructor
     */
    public ItemSequence() {}
           
    /**
     * Accessor method - set the title
     * @param title
     */    
    public void setTitle(String title){
         _title = title;
    }

    /**
     * Accessor method - get the title
     * @return
     */
    public String getTitle(){
         return _title;
    }

    /**
     * Accessor method set the launch
     * @param id
     */
    public void setLaunchUrl(String id){
        //_launchUrl = "../" + id;
        _launchUrl = id;
    }

    /**
     * Accessor method - get the launch
     * @return
     */
    public String getLaunchUrl(){
        return _launchUrl;
    }

    /**
     * Accessor - set the sequence number
     * @param seq
     */
    public void setSequence(int seq){
        _sequence = seq;
    }

    /***
     * Accessor - get the sequence number
     */

    public int getSequence(){
        return _sequence;
    }

    /**
     * Accessor method - to return the full cmi datamodel as a 2-d string array
     * @return
     */
    public String[][] getScoModel(){
        return _scoDataModel.getScoModel();
    }

    /**
     * Method to allow us to update the cmi datamodel (persist it back to disk)
     * @param entries - the name/value pairs
     */
    public void updateClientModel(String[][] entries){       
			_scoDataModel.doLmsCommit(entries);		
    }

    /**
     * Method to set the sco type for this class and if its an
     * sco, then load in the xml cmi model for it.
     * @param scoType string which is the ID of the item
     */
    public void setScoType(String scoType){
        _scormType = scoType;
    }

    /**
     * Method to associate and load in the cmi model for this sco (if it is one)
     */
    public void loadInModel() {
        // if this is a sco then we need to access the
        // cmi data model we created when importing the package
        if (isItemSco()) {
            if (_scoitemID != null) {
                ScoDocument anSco = new ScoDocument();
                anSco.loadDocument(_scoitemID);
                _scoDataModel = anSco;
            }
        }
    }

    /**
     * Method to ascertain if an item is a SCO or not
     * @return true/false
     */
    public boolean isItemSco(){
        if (_scormType.equals(SCORM12_Core.SCO)){
            return true;
        }
        else{
            return false;
        }
    }

    /**
     * Accessor method - to set this items ID
     * @param id
     */
    public void setItemId(String id){
        _scoitemID = id;
    }

    /**
     * Accessor method to allow us to get this item ID
     * @return
     */
    public String getItemId(){
        return _scoitemID;
    }

    /**
     * Accessor method to allow us to return any prerequisites on this item
     * @return
     */
    public String getPrerequisites(){
        return _prerequisites;
    }

    /**
     * Method to allow us to set any prerequisites for this item.
     * If there is a string of chars found in the prerequisistes slot, then
     * we parse it (using the static method in PrerequisiteManager), to ensure
     * that the string is a valid prerequisite string and not just rubbish.
     * If the string does not make sense, then it is junked!
     * @param aprereq
     */
    public void setPrerequisites(String aprereq){
        if (!aprereq.equals("")){
            // get rid of any spaces
            String pre = aprereq.replaceAll(" ", "");
            // check the prerequisites for legality
            if (PrerequisiteManager.isValid(pre)){
                _prerequisites = pre;
                _hasPrerequisites = true;
            }
            // if they are not legal, then junk them...
            else{
                _hasPrerequisites = false;
            }
        }
        else{
            _hasPrerequisites = false;
        }
    }

    /**
     * Accessor method to find aout if there are any prerequisistes attached
     * to this item.
     * @return - boolean
     */
    public boolean hasPrerequisites(){
        return _hasPrerequisites;
    }

}