/**
 *  RELOAD TOOLS
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Paul Sharples
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Paul Sharples
 *  e-mail:   p.sharples@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */
package uk.ac.reload.scormplayer.server.tomcat;

import java.io.File;
import java.util.Hashtable;

import uk.ac.reload.scormplayer.properties.ScormPlayerProperties;
import uk.ac.reload.scormplayer.tomcat.AbstractTomcatHandler;
import uk.ac.reload.scormplayer.tomcat.ITomcatHandler;
import uk.ac.reload.scormplayer.client.generic.prefs.ScormPlayerPrefs;

/**
 * A class used to setup and use the embeddedtomcat class in a particular
 * environment.  In this case it is using the embeddedtomcat class to setup
 * and use it for deploying a scorm web application.
 * @author Paul Sharples.
 * @version $Id: ScormTomcatHandler.java,v 1.2 2004/07/19 16:03:03 ps3com Exp $
 */
public class ScormTomcatHandler extends AbstractTomcatHandler {
    
    /**
     * single instance
     */
    protected static ScormTomcatHandler _sharedInstance;    
    
    /**
     * Our table of contexts to load
     */
    private Hashtable _defaultContexts;
    
    
    private ScormTomcatHandler(){}
    
    /**
     * Get the single instance
     */
    public static ScormTomcatHandler getSharedInstance() {
	    if(_sharedInstance == null) _sharedInstance = new ScormTomcatHandler();
	    return _sharedInstance;
	}
              
   /*
    * Contexts are Web applications themselves. We need to declare which ones
    * the user will have access to by default
    * 
    * @see uk.ac.reload.scormplayer.tomcat.ITomcatHandler#getDefaultContexts()
    */
    public Hashtable getDefaultContexts() {
        if(_defaultContexts == null) {
            _defaultContexts = new Hashtable();
            _defaultContexts.put("", ScormPlayerProperties
                    .getString("ROOT_CONTEXT")
                    + ScormPlayerProperties.getString("WELCOME"));
            _defaultContexts.put(ScormPlayerProperties
                    .getString("WEB_APPLICATION"), ScormPlayerProperties
                    .getString("ROOT_CONTEXT")
                    + ScormPlayerProperties.getString("WEB_APPLICATION"));                           
        }
        return _defaultContexts;
    }

    /*
     * Here we get the port number to run on - obtained from the prefs file.
     * @see uk.ac.reload.scormplayer.tomcat.ITomcatHandler#getPortNumber()
     */
    public int getPortNumber(){
    	return Integer.parseInt(ScormPlayerPrefs.getInstance().getValue(ScormPlayerPrefs.PORT_NUMBER));
    }
     
    
    /*
     * Return a string representing the launch URL of the webapplication
     */
    public String getLaunchURL(String thePackage) {
        boolean showTreeWidget = ScormPlayerPrefs.getInstance()
                .getBooleanValue(ScormPlayerPrefs.SHOW_TREE_WIDGET);
        boolean showNavigation = ScormPlayerPrefs.getInstance()
                .getBooleanValue(ScormPlayerPrefs.SHOW_NAVIGATION);
        boolean enableAutoProgression = ScormPlayerPrefs.getInstance()
                .getBooleanValue(ScormPlayerPrefs.AUTO_PROGRESSION);
        // change URL references with spaces to %20 (for valid URLs)
        thePackage = thePackage.replaceAll(" ", "%20");
        return ITomcatHandler.HTTP_PROTOCOL + getHostName()
                + ":" + getPortNumber()
                + ScormPlayerProperties.getString("WEB_APPLICATION")
                + "/ScormLaunch?scormPackageID=" + thePackage + "&nav="
                + showNavigation + "&tree=" + showTreeWidget + "&ap="
                + enableAutoProgression;        
    }
    
    /*
     * return a string representing the error page
     */
    public String getErrorPage() {
        return ITomcatHandler.HTTP_PROTOCOL + getHostName() + ":"
                + getPortNumber()
                + ScormPlayerProperties.getString("WEB_APPLICATION")
                + ScormPlayerProperties.getString("APPLICATION_PAGES")
                + "/ReloadScormPlayerFiles/error.htm";
    }
    
    
    public String getNoItemsFoundPage() {
        return ITomcatHandler.HTTP_PROTOCOL + getHostName() + ":"
                + getPortNumber()
                + ScormPlayerProperties.getString("WEB_APPLICATION")
                + ScormPlayerProperties.getString("APPLICATION_PAGES")
                + "/ReloadScormPlayerFiles/NoItemsFound.htm";
    }
    
    
    
    /*
     * Return a string representing the LMS frameset.
     */
    public String getLMSPage() {
        return ITomcatHandler.HTTP_PROTOCOL + getHostName() + ":"
                + getPortNumber()
                + ScormPlayerProperties.getString("WEB_APPLICATION")
                + ScormPlayerProperties.getString("APPLICATION_PAGES")
                + "/LMSMain.htm";
    }
    
    /*
     * Return a file reference to a particular SCOs settings file
     */
    public File getScoSettings(String project, String scoID){  
        return new File(getCoursesPath(), project 
                + "/reload-settings/" +  scoID + ".xml");
    }
     
    /*
     * Return a file reference to a courses manifest file
     */
    public File getSettings(String project){  
        return new File(getCoursesPath(), project + "/imsmanifest.xml");
    }
     
    /*
     * Return a file reference to the tracking file
     */
    public File getTrackedSettings(String project){  
        return new File(getCoursesPath(), project + "/reload-settings/reload-settings.xml");
    }
    
    /*
     * Get a list of the "allowed" machines to run this instance of tomcat
     * @see uk.ac.reload.scormplayer.tomcat.ITomcatHandler#getAllowedMachines()
     */
    public String[] getAllowedMachines(){
        String[] allowed = {getHostName()};
        return allowed;
    }
    
    /*
     *  Get the hostname this instance of tomcat is going to runon
     * @see uk.ac.reload.scormplayer.tomcat.ITomcatHandler#getHostName()
     */
    public String getHostName(){
        return ScormPlayerProperties.getString("HOST");
    }
    
    /*
     * Return the server properties info
     */
    public String getServerInfo(){
        return ScormPlayerProperties.getString("SERVER_INFO");
    }
            
    /*  
     * Get the preferences folder
     */ 
    public File getPrefsFolder() {
		return ScormPlayerProperties.getFileProperty("prefs.dir");
	} 
    
    /*
     * Get the folder where tomcat is located
     * @see uk.ac.reload.scormplayer.tomcat.ITomcatHandler#getTomcatPath()
     */
    public File getTomcatPath(){
        return ScormPlayerProperties.getFileProperty("server.dir");
    }
    
    /*
     * Get the folder under the tomcat instalation where the web applications are located
     * @see uk.ac.reload.scormplayer.tomcat.ITomcatHandler#getWebAppsPath()
     */
    public File getWebAppsPath(){
        return ScormPlayerProperties.getFileProperty("webapps.dir");
    }
    
    /*
     * Get the folder where THIS particular web application is located
     */
    public File getScormWebAppPath(){
        return ScormPlayerProperties.getFileProperty("scormwebapp.dir");
    }
    
    /*
     * Get the folder under the web server where the packages are imported
     */
    public File getCoursesPath(){
        return ScormPlayerProperties.getFileProperty("coursepackage.dir");
    }
              
}
