package uk.ac.reload.scormplayer.server.sequence;

import junit.framework.TestCase;
import uk.ac.reload.scormplayer.server.servermodels.SequencerModel;

/**
 * Description  A Junit class to test uk.ac.reload.scormplayer.server.sequence.PrerequisiteManager  
 * 
 * @author Paul Sharples
 * @version $Id: PrerequisiteManagerTest.java,v 1.3 2004/07/20 12:05:53 phillipus Exp $
 */
public class PrerequisiteManagerTest extends TestCase {
		
	public final String NOT_ATTEMPTED = SequencerModel.ITEM_NOT_ATTEMPTED;
	public final String INCOMPLETE = SequencerModel.ITEM_INCOMPLETE;
	public final String COMPLETED = SequencerModel.ITEM_COMPLETED;
	public final String PASSED = SequencerModel.ITEM_PASSED;
	public final String FAILED = SequencerModel.ITEM_FAILED;
	
	/**
	 * TestData - Array of test values.  1st arg is the Sco name, 2nd arg is 
	 * the current state of the sco, 3rd are the prerequisites of this item and 4th is the expected outcome when asking
	 * the canLaunchItem() method if the sco is available to launch.
	 */
	public String[][] testData = {			
			{"S1", NOT_ATTEMPTED, "", "true"}, // PASS  - no prequisites and not yet taken	
			{"S2", COMPLETED, "S1", "false"},	// FAIL - this has been completed								
			{"S3", NOT_ATTEMPTED, "~S1", "true"}, // PASS  - not attempted yet & S1 has not been taken			
			{"S4", NOT_ATTEMPTED, "S3=\"" + NOT_ATTEMPTED + "\"", "true"}, // PASS -  not attempted and S3 had not been attempted			
			{"S5", NOT_ATTEMPTED, "S1|S2|S3|S4", "true"}, // PASS - not attempted and S2 has been completed		
			{"S6", NOT_ATTEMPTED, "S1|~S2|S3|S4", "false"}, // FAIL - not attempted and S1,S3,S4 not attempted and S2 done ie (NOT S2)			
			{"S7", NOT_ATTEMPTED, "~S1|~S2|~S3|~S4", "true"}, // PASS -  S2, S3, S4 are NOTed and so any of them pass			
			{"S8", COMPLETED, "~S1&~S2&~S3", "false"}, // FAIL - can be taken if S1, S2, S3 all not completed, but already taken			
			{"S9", COMPLETED, "~S4&~S5&S6", "false"}, // FAIL - can be taken if S4, S5 not completed & S6 completed, but already taken			
			{"S10", COMPLETED, "S5|S6", "false"},  // FAIL  - can be taken if either S6 or S5 are passed or completed, but already taken			
			{"S11", PASSED, "S5&S6", "false"},  // FAIL  - can be taken if BOTH S6 AND S5 are passed or completed, , but already taken			
			{"S12", PASSED, "", "false"}, // FAIL - no prerequisites, but already taken			
			{"S13", PASSED, "S11=\"" + PASSED + "\"", "false"},  // FAIL -  can only be taken if S11 has been passed, but already taken			
			{"S14", FAILED, "S9=\"" + COMPLETED + "\"", "true"},  // PASS -  can only be taken if S9 has been completed			
			{"S15", FAILED, "S6=\"" + COMPLETED + "\"&S18=\"" + INCOMPLETE + "\"", "false"}, // FAIL - can only be taken if S6 completed AND S21 incomplete			
			{"S16", FAILED, "S15=\"" + FAILED + "\"&S17=\"" + INCOMPLETE + "\"", "true"},  //PASS -  can only be taken if S16 & S17 has been failed
			{"S17", INCOMPLETE, "~S14", "true"}, // PASS - S14 was failed - but is NOTed ~S14
			{"S18", INCOMPLETE, "S98", "true"}, // PASS - items dont exist so ignore
			{"S19", INCOMPLETE, "~S98", "true"}, // PASS - items dont exist so ignore				
			{"S20", INCOMPLETE, "S11=\"" + PASSED + "\"", "true"}, // PASS - can be taken if S11  EQUAL to PASSED
			{"S21", INCOMPLETE, "S11<>\"" + PASSED + "\"", "false"}, // FAIL - can not be taken if S11 NOT EQUAL to PASSED
			{"S22", INCOMPLETE, "S1=\""+ NOT_ATTEMPTED + "\"", "true"}, // PASS - S1 has not been attempted
			{"S23", INCOMPLETE, "~S1=\"" + NOT_ATTEMPTED + "\"", "false"}, // FAIL -  same as above but NOTed
			{"S24", INCOMPLETE, "S1=\"" + NOT_ATTEMPTED + "\"&S11<>\"" + PASSED + "\"", "false"}, // FAIL - can be taken if S10 NOT EQUAL to PASSED
			{"S25", INCOMPLETE, "S1=\"" + NOT_ATTEMPTED + "\"&S11=\"" + PASSED + "\"", "true"}, // PASS - can be taken if S10 EQUAL to FAILED
			{"S26", INCOMPLETE, "S1<>\"" + COMPLETED + "\"", "true"}, // PASS - S1 doesnt equal completed			
			{"S27", NOT_ATTEMPTED, "S11<>\"" + FAILED + "\"", "true"}, // PASS - S1 does not equal FAILED
			{"S28", FAILED, "(S1<>\""+ COMPLETED + "\")&((S5|S6|S7)&(s11))", "true"}, // PASS 		
			{"S29", INCOMPLETE, "(S1<>\""+ NOT_ATTEMPTED + "\")&((S5|S6|S7)&(s11))", "true"}, // PASS  - S1 has been completed
			{"S30", INCOMPLETE, "~S1&S2=\"" + COMPLETED + "\"&~S3&~S4&~S5&~S6&~S7&S8&S9&S10&S11&S12&S13&S14<>\"" + PASSED + "\"", "true"}, // PASS  
	        {"S31", INCOMPLETE, "~S1&S2=\"" + COMPLETED + "\"&~S3&~S4&~S5&~S6&~S7&S8&S9&S10&S11&S12&S13&S14=\"" + PASSED + "\"", "false"} // FAIL
	};
	
    /**
     * The path to test data
     */
    private String pathTestData;
    
	//	---------------------------------------------------------------------------------------------

    /** 
     * Set some paths to properties file and test folder for testing
     */
    protected void setUp() throws Exception {
        // See if we have a testdata dir set from an Ant script
        pathTestData = System.getProperty("testdata.dir");
        if(pathTestData == null) {
            // Set a default
            pathTestData = "../junit-tests";
        }

        // Set Properties file to test properties
		System.setProperty("scormplayer.properties.file", "uk.ac.reload.scormplayer.testproperties.rb");
    }

	//	---------------------------------------------------------------------------------------------

    /**
	 * Utility method to setup an instance of PrequisiteManager with test data
	 * @param testingData 2d array of test data
	 * @return a PrerequisiteManager instance
	 * @throws Exception
	 */
	protected PrerequisiteManager getManagerWithTestData(String[][] testingData) throws Exception {
		PrerequisiteManager prerequisiteManager = new PrerequisiteManager();
		for (int i = 0; i < testingData.length; i++) {
			prerequisiteManager.updatePrerequisites(testingData[i][0],
					testingData[i][1], false);
		}
		return prerequisiteManager;
	}

	//	---------------------------------------------------------------------------------------------
	
	/**
	 * Test each test in the tastdata array and see if the method "canLaunchItem" returns what it is expected to
	 */
	public void testCanLaunchItem() throws Exception {
		PrerequisiteManager prerequisiteManager = getManagerWithTestData(testData);
		boolean result;
		
		for(int i = 0; i < testData.length; i++) {
			result = prerequisiteManager.canLaunchItem(testData[i][0], testData[i][2]);
			assertEquals("incorrect boolean result on item " + testData[i][0], Boolean.valueOf(testData[i][3]).booleanValue(), result);
		}
	}
		
	//	---------------------------------------------------------------------------------------------
	
	/*
	 *  IsValid does first level checking - making sure no illegal chars & correct 
	 *  number of brackets, but does not check for meaning at this stage.
	 */
	
	/**
	 * Test a simple case of one item.
	 */
	public void testIsValid1(){
		String prereqTest ="S1";
		assertTrue("Prequisite is in correct format", PrerequisiteManager.isValid(prereqTest));
	}
	
	/**
	 * Test a case of NOT simple item
	 */
	public void testIsValid2(){
		String prereqTest ="~S1";
		assertTrue("Prequisite is in correct format", PrerequisiteManager.isValid(prereqTest));
	}
	
	/**
	 * Test illegal char @
	 */	
	public void testIsValid3(){
		String prereqTest ="@S1";
		assertFalse("Prequisite is not in correct format", PrerequisiteManager.isValid(prereqTest));
	}
	
	/**
	 * Test an AND case
	 */		
	public void testIsValid4(){
		String prereqTest ="S1&S2";
		assertTrue("Prequisite is in correct format", PrerequisiteManager.isValid(prereqTest));
	}
	
	/**
	 * Test an OR case
	 */	
	public void testIsValid5(){
		String prereqTest ="S1|S2";
		assertTrue("Prequisite is in correct format", PrerequisiteManager.isValid(prereqTest));
	}
	
	/**
	 * check with correct number of double quotes	 
	 */
	public void testIsValid6(){
		String prereqTest ="S1=\"completed\"";		
		assertTrue("2. Prequisite is in correct format", PrerequisiteManager.isValid(prereqTest));
	}
	
	/**
	 * Test with single quotes (illegal)
	 */
	public void testIsValid7(){
		String prereqTest ="S1='completed'";
		assertFalse("Prequisite is not in correct format", PrerequisiteManager.isValid(prereqTest));
	}
	
	/**
	 * Check with odd number of double quotes
	 */
	public void testIsValid8(){
		String prereqTest ="S1=\"comp\"leted\"";
		assertFalse("Prequisite is not in correct format", PrerequisiteManager.isValid(prereqTest));
	}
	
	/**
	 * Test an AND case with spaces
	 */		
	public void testIsValid9(){
		String prereqTest ="S1 & S2 & S3 & S4";
		assertTrue("Prequisite is in correct format", PrerequisiteManager.isValid(prereqTest));
	}
	
	/**
	 * Test an AND case with brackets 7 spaces
	 */		
	public void testIsValid10(){
		String prereqTest ="S1  & (S2 | S3 | S4)";
		assertTrue("Prequisite is in correct format", PrerequisiteManager.isValid(prereqTest));
	}
	
	/**
	 * Test an AND case with illegal ODD number of brackets
	 */		
	public void testIsValid11(){
		String prereqTest ="S1  & (S2 | S3 | S4))";
		assertFalse("Prequisite is not in correct format", PrerequisiteManager.isValid(prereqTest));
	}
	
	/**
	 * Test an AND case with NOT operator
	 */		
	public void testIsValid12(){
		String prereqTest ="S1  & (~S2 & ~S3 & ~S4)";
		assertTrue("Prequisite is in correct format", PrerequisiteManager.isValid(prereqTest));
	}
		
	/**
	 * Test with lots of brackets
	 */		
	public void testIsValid13(){
		String prereqTest ="S8 | (S9|(S1&(~S2&~S3&~S4)))";
		assertTrue("Prequisite is in correct format", PrerequisiteManager.isValid(prereqTest));
	}
	
	/**
	 * Test with NOT EQUALs operator
	 */		
	public void testIsValid14(){
		String prereqTest ="S1<>\"completed\" & S2<>=\"passed\"";
		assertTrue("Prequisite is in correct format", PrerequisiteManager.isValid(prereqTest));
	}
	
	/**
	 * Test for sets   {}  (not yet supported)
	 */		
	public void testIsValid15(){
		String prereqTest ="3*{S1,S2,S3}";
		assertFalse("Prequisite is not in correct format", PrerequisiteManager.isValid(prereqTest));
	}	
	
	//	---------------------------------------------------------------------------------------------
	
	/**
	 * simple test to make sure item exist 
	 */
	public void testdoesItemExist1() throws Exception {		
		assertTrue("Incorrect value returned Item should exist", getManagerWithTestData(testData).doesItemExist("S1"));
	}

	/**
	 * simple test to make sure item doesn't exist 
	 */
	public void testdoesItemExist2() throws Exception {		
		assertFalse("Incorrect value returned Item should not exist", getManagerWithTestData(testData).doesItemExist("Z1"));
	}	
	
	//	---------------------------------------------------------------------------------------------
	
	/**
	 * Make sure that the correct status is returned for each item
	 */
	public void testgetStatus() throws Exception {	    
	    PrerequisiteManager prerequisiteManager = getManagerWithTestData(testData);
		for (int i = 0; i < testData.length; i++) {
			String status = prerequisiteManager.getStatus(testData[i][0]);
			assertEquals("Incorrect status on item " + testData[i][0], testData[i][1], status);
		}
	}
	
	//	---------------------------------------------------------------------------------------------
	
	/**
	 * Test that correct boolean state is returned - true if "completed" or "passed" otherwise false
	 */
	public void testcheckStatus() throws Exception {
	    PrerequisiteManager prerequisiteManager = getManagerWithTestData(testData);
		for(int i = 0; i < testData.length; i++) {
			boolean status = prerequisiteManager.checkStatus(testData[i][0]);
			boolean expectedResult;
			if(testData[i][1]== COMPLETED || testData[i][1]== PASSED) {
			    expectedResult = true;
			}
			else {
			    expectedResult = false;
			}
			assertEquals("Incorrect completed/passed status on item: " + testData[i][0], expectedResult, status);
		}
	}
	
	//	---------------------------------------------------------------------------------------------	
}
