/**
 *  RELOAD TOOLS
 *
 *  Copyright (c) 2004 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.xindice.embedded;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;

import org.apache.xindice.client.xmldb.services.CollectionManager;
import org.apache.xindice.core.DBException;
import org.apache.xindice.util.XindiceException;
import org.apache.xindice.xml.dom.DOMParser;
import org.jdom.Document;
import org.jdom.JDOMException;
import org.xmldb.api.DatabaseManager;
import org.xmldb.api.base.Collection;
import org.xmldb.api.base.Resource;
import org.xmldb.api.base.ResourceSet;
import org.xmldb.api.base.XMLDBException;
import org.xmldb.api.modules.XMLResource;
import org.xmldb.api.modules.XPathQueryService;

import uk.ac.reload.jdom.XMLUtils;




/**
 * XindiceEmbeddedDatabaseManager
 * <br>
 * Manages the Xindice Database Embedded version.
 * 
 * @author Phillip Beauvoir
 * @version $Id: XindiceEmbeddedDatabaseManager.java,v 1.4 2004/06/21 13:37:33 phillipus Exp $
 */
public class XindiceEmbeddedDatabaseManager {

    /**
     * The root system DB
     */
    public static String COLLECTION_ROOT_URI = "xmldb:xindice-embed:///";
    
    /**
     * DB Driver
     */
    public static String driver = "org.apache.xindice.client.xmldb.DatabaseImpl";
    
    /**
     * The default Database Root Name
     */
    public static String DEFAULT_DB_ROOTNAME = "db";
    
    /**
     * The XMLDB API Database
     */
    private org.xmldb.api.base.Database _database;
    
    /**
     * The Database Root URI
     */
    private String DB_ROOT_URI;
    
    /**
     * The Database Name
     */
    private String _dbName;
    
    
    /**
     * Initialise The Xindice DB and set the DB home folder and use the default "db" Root Name.
     * Only call this once.
     * @param dbHome The file path location of the root of the database
     * @return The Database
     * @throws XindiceException
     */
    public org.xmldb.api.base.Database registerDatabase(String dbHome) throws XindiceException {
        _dbName = DEFAULT_DB_ROOTNAME;
        
        DB_ROOT_URI = COLLECTION_ROOT_URI + _dbName + "/";
        
        register();
        
        try {
            if(dbHome != null) {
                _database.setProperty("db-home", dbHome);
            }
        }
        catch(XMLDBException ex) {
            throw new XindiceException(ex);
        }
         
        return _database;
    }
    
    /**
     * Initialise The Xindice DB using a Config File.
     * Only call this once.
     * @param configFile The file location of the Config file
     * @param dbName The name of the database to use in the config file
     * @return The Database
     * @throws XindiceException
     * @throws JDOMException
     * @throws IOException
     */
    public org.xmldb.api.base.Database registerDatabase(File configFile, String dbName) throws XindiceException {
        _dbName = dbName;
        
        DB_ROOT_URI = COLLECTION_ROOT_URI + _dbName + "/";

        register();
        
        try {
            if(configFile != null) {
                _database.setProperty("configuration", configFile.getPath());
            }
        }
        catch(XMLDBException ex) {
            throw new XindiceException(ex);
        }
        
        return _database;
    }
    
    /**
     * Register the Database
     * @throws XindiceException
     */
    protected void register() throws XindiceException {
        if(_database == null) {
            try {
                Class driverClass = Class.forName(driver);
                _database = (org.xmldb.api.base.Database)driverClass.newInstance();
                DatabaseManager.registerDatabase(_database);
            }
            catch(Exception ex) {
                throw new XindiceException(ex);
            }
        }
    }
    
    /**
     * Close the Database
     * @throws DBException
     */
    public void close() throws DBException {
        org.apache.xindice.core.Database db = org.apache.xindice.core.Database.getDatabase(_dbName);
        if(db != null) {
            db.close();
        }
    }

    /**
     * Get the Root Database "Collection" - i.e a Database.  If it doesn't exist it will be created.
     * You must call registerDatabase() first before calling this!
     * @return The Collection or null if not found
     * @throws XMLDBException if there is a problem getting the Collection
     * @throws XindiceException if registerDatabase() has not been correctly called first
     */
    public Collection getRootCollection() throws XMLDBException, XindiceException {
        if(_database == null || DB_ROOT_URI == null) {
            throw new XindiceException("Must call registerDatabase() first");
        }
        
        return DatabaseManager.getCollection(DB_ROOT_URI);
    }
    
    /**
     * Get a Database "Collection" - i.e a Database.  The Collection must be created first in order to exist.
     * You must call registerDatabase() first before calling this!
     * @param collectionName The Name of the Collection
     * @return The Collection, or null if not found
     * @throws XMLDBException if there is a problem getting the Collection
     * @throws XindiceException if registerDatabase() has not been correctly called first
     */
    public Collection getCollection(String collectionName) throws XMLDBException, XindiceException {
        if(_database == null|| DB_ROOT_URI == null) {
            throw new XindiceException("Must call registerDatabase() first");
        }
        
        return DatabaseManager.getCollection(DB_ROOT_URI + collectionName);
    }

    /**
     * Create a new Collection in the Database.
     * @param collectionName The name of the Collection
     * @return The new Collection Database
     * @throws XMLDBException if the Collection already exists
     * @throws XindiceException
     */
    public Collection createCollection(String collectionName) throws XMLDBException, XindiceException {
        // Get the Root Collection
        Collection rootCollection = getRootCollection();
        if(rootCollection == null) {
            throw new XindiceException("Root Collection was null.");
        }
        
        CollectionManager service = (CollectionManager) rootCollection.getService("CollectionManager", "1.0");
        
        // Build up the Collection XML configuration.
        String collectionConfig =
            "<collection compressed=\"true\" name=\"" + collectionName + "\">" +
            	"<filer class=\"org.apache.xindice.core.filer.BTreeFiler\" gzip=\"true\"/>" +
            	"<indexes />" +
            "</collection>";
        
        Collection newColllection = service.createCollection(collectionName, DOMParser.toDocument(collectionConfig));
        rootCollection.close();
        return newColllection;
    }
    
    /**
     * Delete a Collection
     * @param collectionName The name of the Collection
     * @throws XMLDBException
     * @throws XindiceException
     */
    public void deleteCollection(String collectionName) throws XMLDBException, XindiceException {
        Collection collection = getCollection(collectionName);
        if(collection == null) {
            throw new XindiceException("Collection <" + collectionName + "> was null.");
        }
        // Get the Root Collection and CollectionManager
        Collection rootCollection = getRootCollection();
        if(rootCollection == null) {
            throw new XindiceException("Root Collection was null.");
        }
        CollectionManager service = (CollectionManager) rootCollection.getService("CollectionManager", "1.0");
        service.dropCollection(collectionName);
    }
    
    /**
     * Add an XML File to the Collection
     * @param collectionName The Name of the Collection to add to 
     * @param file The XML File to add
     * @param resourceName The Resource Name to give to the resource that is added.  If null a unique id
     * will be created by Xindice
     * @return The XMLResource created
     * @throws XMLDBException
     * @throws XindiceException
     * @throws IOException
     */
    public XMLResource addResourceAsFile(String collectionName, File file, String resourceName) throws XMLDBException, XindiceException, IOException {
        // Read File into String
        FileInputStream in = new FileInputStream(file);
        byte[] fileBuffer = new byte[(int)file.length()];
        in.read(fileBuffer);
        in.close();
        String data = new String(fileBuffer);
        
        return addResourceAsString(collectionName, data, resourceName);
    }
    
    /**
     * Add an XML String to the Collection
     * @param collectionName The Name of the Collection to add to
     * @param data The XML String
     * @param resourceName The Resource Name to give to the resource that is added.  If null a unique id
     * will be created by Xindice
     * @return The XMLResource created or null if not successful
     * @throws XMLDBException
     * @throws XindiceException
     */
    public XMLResource addResourceAsString(String collectionName, String data, String resourceName) throws XMLDBException, XindiceException {
        Collection collection = getCollection(collectionName);
        if(collection == null) {
            throw new XindiceException("Collection <" + collectionName + "> was null.");
        }

        XMLResource resource = (XMLResource)collection.createResource(resourceName, XMLResource.RESOURCE_TYPE);
        resource.setContent(data);
        collection.storeResource(resource);
        collection.close();
        return resource;
    }
    
    /**
     * Add a JDOM Document to the Collection
     * @param collectionName The Name of the Collection to add to
     * @param doc The JDOM Document
     * @param resourceName The Resource Name to give to the resource that is added.  If null a unique id
     * will be created by Xindice
     * @return The XMLResource created
     * @throws XMLDBException
     * @throws XindiceException
     * @throws IOException
     */
    public XMLResource addResourceAsJDOM(String collectionName, Document doc, String resourceName) throws XMLDBException, XindiceException, IOException {
        String data = XMLUtils.write2XMLString(doc);
        return addResourceAsString(collectionName, data, resourceName);
    }

    /**
     * Delete a Resource from the Collection
     * @param collectionName The Name of the Collection to delete from
     * @param resourceName The name of the Resource to delete
     * @throws XMLDBException
     * @throws XindiceException
     */
    public void deleteResource(String collectionName, String resourceName) throws XMLDBException, XindiceException {
        Collection collection = getCollection(collectionName);
        if(collection == null) {
            throw new XindiceException("Collection <" + collectionName + "> was null.");
        }
        Resource resource = collection.getResource(resourceName);
        collection.removeResource(resource);
        collection.close();
    }

    /**
     * Retrieve a Resource from the Collection as an XML String
     * @param collectionName The Name of the Collection to retrieve from
     * @param resourceName The name of the Resource to retrieve
     * @return The XML String or null if unsuccessful
     * @throws XMLDBException
     * @throws XindiceException
     */
    public String retrieveResourceAsString(String collectionName, String resourceName) throws XMLDBException, XindiceException {
        Collection collection = getCollection(collectionName);
        if(collection == null) {
            throw new XindiceException("Collection <" + collectionName + "> was null.");
        }

        XMLResource xmlResource = (XMLResource)collection.getResource(resourceName);
        collection.close();

        return xmlResource == null ? null : (String)xmlResource.getContent();
    }

    /**
     * Retrieve a Resource from the Collection as a JDOM Document
     * @param collectionName The Name of the Collection to retrieve from
     * @param resourceName The name of the Resource to retrieve
     * @return The JDOM Document
     * @throws XMLDBException
     * @throws XindiceException
     * @throws IOException
     * @throws JDOMException
     */
    public Document retrieveResourceAsJDOM(String collectionName, String resourceName) throws XMLDBException,
    														XindiceException, IOException, JDOMException {
        // Get as String out of DB
        String data = retrieveResourceAsString(collectionName, resourceName);
        // Convert to JDOM Doc
        return XMLUtils.readXMLString(data);
    }

    /**
     * Query the Database using XPath
     * @param collectionName The Name of the Collection to retrieve from
     * @param xpath The XPath query
     * @return The ResourceSet of Resources found or null if unsuccessful
     * @throws XMLDBException
     * @throws XindiceException
     */
    public ResourceSet queryResource(String collectionName, String xpath) throws XMLDBException, XindiceException {
        Collection collection = getCollection(collectionName);
        if(collection == null) {
            throw new XindiceException("Collection <" + collectionName + "> was null.");
        }

        XPathQueryService service = (XPathQueryService)collection.getService("XPathQueryService", "1.0");
        ResourceSet resourceSet = service.query(xpath);
        collection.close();
        return resourceSet;
    }
}
