
package uk.ac.reload.xindice.embedded;

import java.io.File;

import junit.framework.TestCase;

import org.apache.xindice.util.XindiceException;
import org.jdom.Document;
import org.jdom.Element;
import org.xmldb.api.base.*;
import org.xmldb.api.modules.XMLResource;

import uk.ac.reload.jdom.XMLUtils;
import uk.ac.reload.testsupport.FileSupport;


/**
 * XindiceEmbeddedDatabaseManagerTest
 * 
 * @author Phillip Beauvoir
 * @version $Id: XindiceEmbeddedDatabaseManagerTest.java,v 1.3 2004/07/06 13:14:23 phillipus Exp $
 */
public class XindiceEmbeddedDatabaseManagerTest extends TestCase {
    
    /**
     * The Resource Name of the XML File
     */
    protected static final String RESOURCE_NAME = "address1";
    
    /**
     * The Collection Database name
     */
    protected final static String COLLECTION_NAME = "test-database";

    /**
     * XML File 
     */
    protected static File xmlFile;

    /**
     * Config File 
     */
    protected static File configFile;

    /**
     * The XindiceEmbeddedDatabaseManager
     */
    protected XindiceEmbeddedDatabaseManager xindiceEmbeddedDatabaseManager;
    
    /**
     * The actual Database
     */
    protected Database database;
    
    String workingdir;
    
    // ---------------------------------------------------------------------------------------------

    static {
        // Highest level of logging
        System.setProperty("org.apache.commons.logging.Log", "org.apache.commons.logging.impl.SimpleLog");
        System.setProperty("org.apache.commons.logging.simplelog.defaultlog", "fatal"); 
    }

    protected void setUp() throws Exception {
        // See if we have a working dir set from an Ant script
        workingdir = System.getProperty("working.dir");
        if(workingdir == null) {
            workingdir = ".";
        }
        xmlFile = new File(workingdir, "testdata/address1.xml");
        configFile = new File(workingdir, "testdata/config.xml");
    }
    
    /** 
     * Tidy up
     */
    protected void tearDown() throws Exception {
        xindiceEmbeddedDatabaseManager.close();
        FileSupport.deleteFolder(FileSupport.getMainTestFolder());
        FileSupport.deleteFolder(new File(workingdir, "testdata/~xindicetest~"));
    }
    
    /**
     * Register the Database using the config file
     * @param dbName The name of the database to use in the config file
     * @throws Exception
     */
    private void registerDatabase(String dbName) throws Exception {
        xindiceEmbeddedDatabaseManager = new XindiceEmbeddedDatabaseManager();
        database = xindiceEmbeddedDatabaseManager.registerDatabase(configFile, dbName);
        assertNotNull("Database is null", database);
    }
    
    /**
     * Register the Database using the default settings in a folder
     * @throws Exception
     */
    private void registerDefaultDatabase() throws Exception {
        File tmpFolder = FileSupport.getMainTestFolder();
        xindiceEmbeddedDatabaseManager = new XindiceEmbeddedDatabaseManager();
        database = xindiceEmbeddedDatabaseManager.registerDatabase(tmpFolder.getPath());
        assertNotNull("Database is null", database);

        Collection collection = xindiceEmbeddedDatabaseManager.createCollection(COLLECTION_NAME);
        assertNotNull("Collection was null", collection);
        assertEquals("Collection name wrong", COLLECTION_NAME, collection.getName());
        collection.close();
    }
    
    
    // ---------------------------------------------------------------------------------------------

    /**
     * Ensure the Root Collection is automatically created
     */
    public void testGetRootCollection() throws Exception {
        File tmpFolder = FileSupport.getMainTestFolder();
        xindiceEmbeddedDatabaseManager = new XindiceEmbeddedDatabaseManager();
        database = xindiceEmbeddedDatabaseManager.registerDatabase(tmpFolder.getPath());
        assertNotNull("Database is null", database);

        Collection collection = xindiceEmbeddedDatabaseManager.getRootCollection();
        assertNotNull("Collection was null", collection);
        assertEquals("Collection name wrong", XindiceEmbeddedDatabaseManager.DEFAULT_DB_ROOTNAME, collection.getName());
        collection.close();
    }
    
    /**
     * Ensure an Exception is thrown if the Database is not registered before getting the Root Collection
     */
    public void testGetRootCollection_NotRegistered() {
        File tmpFolder = FileSupport.getMainTestFolder();
        xindiceEmbeddedDatabaseManager = new XindiceEmbeddedDatabaseManager();

        try {
            Collection collection = xindiceEmbeddedDatabaseManager.getRootCollection();
            fail("Should have thrown XindiceException");
        }
        catch(XindiceException ex) {
            assertTrue(true);
        }
        catch(XMLDBException ex) {
            fail("Should not throw XMLDBException");
        }
    }

    // ---------------------------------------------------------------------------------------------
    
    /**
     * Ensure we can re-open the Collection
     */
    public void testGetCollection() throws Exception {
        registerDefaultDatabase();
        
        Collection collection = xindiceEmbeddedDatabaseManager.getCollection(COLLECTION_NAME);
        assertNotNull("Collection was null", collection);
        assertEquals("Collection name wrong", COLLECTION_NAME, collection.getName());
        collection.close();
    }

    /**
     * Check Collection is null if it hasn't been created
     */
    public void testGetCollection_Null() throws Exception {
        File tmpFolder = FileSupport.getMainTestFolder();
        xindiceEmbeddedDatabaseManager = new XindiceEmbeddedDatabaseManager();
        database = xindiceEmbeddedDatabaseManager.registerDatabase(tmpFolder.getPath());
        assertNotNull("Database is null", database);

        Collection collection = xindiceEmbeddedDatabaseManager.getCollection(COLLECTION_NAME);
        assertNull("Collection should be null", collection);
    }
    
    // ---------------------------------------------------------------------------------------------

    /**
     * Check that an Exception is thrown if the Collection already exists
     */
    public void testCreateCollection() throws Exception {
        registerDefaultDatabase();
        try {
            Collection collection = xindiceEmbeddedDatabaseManager.createCollection(COLLECTION_NAME);
            fail("Should have thrown XMLDBException");
        }
        catch(XMLDBException ex) {
            assertTrue(true);
        }
        catch(XindiceException ex) {
            fail("Should not throw XindiceException");
        }
    }
    
    // ---------------------------------------------------------------------------------------------

    /**
     * Create a Collection using the config file
     */
    public void testCreateCollection_ConfigFile1() throws Exception {
        registerDatabase("dbroot");
        
        Collection collection = xindiceEmbeddedDatabaseManager.createCollection(COLLECTION_NAME);
        assertNotNull("Collection was null", collection);
        assertEquals("Collection name wrong", COLLECTION_NAME, collection.getName());
    }
    
    /**
     * Create a Collection using the config file
     */
    public void testCreateCollection_ConfigFile2() throws Exception {
        registerDatabase("dbroot2");
        
        Collection collection = xindiceEmbeddedDatabaseManager.createCollection(COLLECTION_NAME);
        assertNotNull("Collection was null", collection);
        assertEquals("Collection name wrong", COLLECTION_NAME, collection.getName());
    }

    /**
     * Check that an Exception is thrown if the db name is not in the config file
     */
    public void testCreateCollection_ConfigFile3() throws Exception {
        registerDatabase("nonsense");
        
        try {
            Collection collection = xindiceEmbeddedDatabaseManager.createCollection(COLLECTION_NAME);
            fail("Should have thrown Exception");
        }
        catch(Exception ex) {
            assertTrue(true);
        }
    }

    //  ---------------------------------------------------------------------------------------------
    
    /**
     * Check we can delete a Collection
     */
    public void testDeleteCollection() throws Exception {
        registerDefaultDatabase();
        
        Collection collection = xindiceEmbeddedDatabaseManager.getCollection(COLLECTION_NAME);
        assertNotNull("Collection was null", collection);
        assertEquals("Collection name wrong", COLLECTION_NAME, collection.getName());

        xindiceEmbeddedDatabaseManager.deleteCollection(COLLECTION_NAME);
        
        collection = xindiceEmbeddedDatabaseManager.getCollection(COLLECTION_NAME);
        assertNull("Collection should be null", collection);
    }
    
    //  ---------------------------------------------------------------------------------------------

    /**
     * Check we can add an xml file as a Resource
     */
    public void testAddResourceAsFile() throws Exception {
        registerDefaultDatabase();
        
        XMLResource resource = xindiceEmbeddedDatabaseManager.addResourceAsFile(COLLECTION_NAME, xmlFile, RESOURCE_NAME);
        assertNotNull("XMLResource was null", resource);
        assertEquals("XMLResource ID wrong", RESOURCE_NAME, resource.getId());
    }
    
    /**
     * Check that a Resource ID (name) is created if one is not provided
     */
    public void testAddResourceAsFile_NoResourceName() throws Exception {
        registerDefaultDatabase();
        
        XMLResource resource = xindiceEmbeddedDatabaseManager.addResourceAsFile(COLLECTION_NAME, xmlFile, null);
        assertNotNull("XMLResource was null", resource);
        assertNotNull("XMLResource ID wrong", resource.getId());
    }

    //  ---------------------------------------------------------------------------------------------

    /**
     * Test that we can add an XML String as a Resource
     */
    public void testAddResourceAsString() throws Exception {
        registerDefaultDatabase();

        String resourceName = "ponk";
        String data = "<root/>";
        
        XMLResource resource = xindiceEmbeddedDatabaseManager.addResourceAsString(COLLECTION_NAME, data, resourceName);
        assertNotNull("XMLResource was null", resource);
        assertEquals("XMLResource ID wrong", resourceName, resource.getId());
        
        String content = xindiceEmbeddedDatabaseManager.retrieveResourceAsString(COLLECTION_NAME, resourceName);
        String expected = "<?xml version=\"1.0\"?>\n<root />";
        assertEquals("Content was wrong", expected, content);
    }
    
    //  ---------------------------------------------------------------------------------------------

    /**
     * Test that we can add an XML String as a JDOM Document
     */
    public void testAddResourceAsJDOM() throws Exception {
        registerDefaultDatabase();
        
        Document doc = XMLUtils.readXMLFile(xmlFile);
        XMLResource resource = xindiceEmbeddedDatabaseManager.addResourceAsJDOM(COLLECTION_NAME, doc, RESOURCE_NAME);
        assertNotNull("XMLResource was null", resource);
        assertEquals("XMLResource ID wrong", RESOURCE_NAME, resource.getId());
    }
    
    //  ---------------------------------------------------------------------------------------------

    /**
     * Test that we can delete a Resource
     */
    public void testDeleteResource() throws Exception {
        registerDefaultDatabase();

        XMLResource resource = xindiceEmbeddedDatabaseManager.addResourceAsFile(COLLECTION_NAME, xmlFile, RESOURCE_NAME);
        assertNotNull("XMLResource was null", resource);
        assertEquals("XMLResource ID wrong", RESOURCE_NAME, resource.getId());

        xindiceEmbeddedDatabaseManager.deleteResource(COLLECTION_NAME, RESOURCE_NAME);

        String content = xindiceEmbeddedDatabaseManager.retrieveResourceAsString(COLLECTION_NAME, RESOURCE_NAME);
        assertNull("Content should be null", content);
    }

    //  ---------------------------------------------------------------------------------------------
    
    /**
     * Test that we can retrieve a Resource as a String
     */
    public void testRetrieveResourceAsString() throws Exception {
        registerDefaultDatabase();

        XMLResource resource = xindiceEmbeddedDatabaseManager.addResourceAsFile(COLLECTION_NAME, xmlFile, RESOURCE_NAME);
        assertNotNull("XMLResource was null", resource);
        assertEquals("XMLResource ID wrong", RESOURCE_NAME, resource.getId());

        String content = xindiceEmbeddedDatabaseManager.retrieveResourceAsString(COLLECTION_NAME, RESOURCE_NAME);
        assertNotNull("Content was null", content);
    }

    //  ---------------------------------------------------------------------------------------------
    
    /**
     * Test that we can retrieve a Resource as a JDOM Document
     */
    public void testRetrieveResourceAsJDOM() throws Exception {
        registerDefaultDatabase();

        XMLResource resource = xindiceEmbeddedDatabaseManager.addResourceAsFile(COLLECTION_NAME, xmlFile, RESOURCE_NAME);
        assertNotNull("XMLResource was null", resource);
        assertEquals("XMLResource ID wrong", RESOURCE_NAME, resource.getId());

        Document doc = xindiceEmbeddedDatabaseManager.retrieveResourceAsJDOM(COLLECTION_NAME, RESOURCE_NAME);
        assertNotNull("Document was null", doc);
    }

    //  ---------------------------------------------------------------------------------------------

    /**
     * Test that we can query an XML Resource using XPath
     */
    public void testQueryResource() throws Exception {
        registerDefaultDatabase();

        XMLResource resource = xindiceEmbeddedDatabaseManager.addResourceAsFile(COLLECTION_NAME, xmlFile, RESOURCE_NAME);
        assertNotNull("XMLResource was null", resource);
        assertEquals("XMLResource ID wrong", RESOURCE_NAME, resource.getId());

        String xpath = "//person[fname='John']";

        ResourceSet resourceSet = xindiceEmbeddedDatabaseManager.queryResource(COLLECTION_NAME, xpath);
        assertNotNull("ResourceSet was null", resourceSet);
        ResourceIterator resourceIterator = resourceSet.getIterator();

        while(resourceIterator.hasMoreResources()) {
            Resource res = resourceIterator.nextResource();
            assertNotNull("Resource was null", res);
            //System.out.println((String) res.getContent());
        }
    }

    //  ---------------------------------------------------------------------------------------------

    /**
     * Test that we can retrieve a Resource as a JDOM Document, modify it and put it back.
     */
    public void testRetrieve_Modify_And_Add_Resource_With_JDOM() throws Exception {
        registerDefaultDatabase();

        XMLResource resource = xindiceEmbeddedDatabaseManager.addResourceAsFile(COLLECTION_NAME, xmlFile, RESOURCE_NAME);
        assertNotNull("XMLResource was null", resource);
        assertEquals("XMLResource ID wrong", RESOURCE_NAME, resource.getId());

        // Get as JDOM Doc
        Document doc = xindiceEmbeddedDatabaseManager.retrieveResourceAsJDOM(COLLECTION_NAME, RESOURCE_NAME);
        assertNotNull("Document was null", doc);
        
        // Change Element in JDOM
        Element element = doc.getRootElement().getChild("fname");
        assertNotNull("Element was null", element);
        element.setText("Fido");
        
        // Put in DB
        xindiceEmbeddedDatabaseManager.addResourceAsJDOM(COLLECTION_NAME, doc, RESOURCE_NAME);
        
        // Get it back again
        String content = xindiceEmbeddedDatabaseManager.retrieveResourceAsString(COLLECTION_NAME, RESOURCE_NAME);
        assertNotNull("Content was null", content);
        assertTrue("Element not changed", content.indexOf("<fname>Fido</fname>") != -1);
    }
    
    //  ---------------------------------------------------------------------------------------------

}



